/*
 * Project name:
     MP3 (Demonstration of on-board audio module)
     Now for FAT32
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20120810:
       - initial release (FJ);
     20200920:
       - Fat 32 Mark Joyce 2020
 * Description:
     This project demonstrates communication with VS1011E mp3 codec.
     Program reads one mp3 file from SDHC and sends it to VS1011E for decoding
     and playing. SDHC and MP3_SCI share Hardware SPI module.
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7 - ac:MP3
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.0000MHz
     Ext. Modules:    EasyTFT display - ac:EasyTFT
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Turn on SPI and AUDIO switches at switch SW13. (board specific)
     - Turn on microSD card switches at SW14. (board specific)
     - Turn on TFT backlight on SW11.1. (board specific)
     - Put PORTA switches in the pull-down position and turn on Button Press
       Level switch SW10 for PORTA&C.
     - Use RA1 and RA5 to increase volume for left and right channels,
       RA0 and RA4 do decrease volume for left and right channels, respecively.
     - MP3 file should have the following name : "hello_my_name_is_mark.mp3.mp3".
     Attention! Trap for new players.
     - ! The audio jack on the decoder is not compatable with all plugs (headphones etc), you may
          have to pull it out a bit to make a connection.
 */
#include "MP3_routines.h"
#include "resources.h"

// MCU initialization
void Init() {
  AD1PCFG = 0xFFFF;            // Configure AN pins as digital I/O
  JTAGEN_bit = 0;              // Disable JTAG
  TFT_BLED_Direction = 0;      // Set TFT backlight pin as output
  TFT_Init_ILI9341_8bit(320, 240);// Initialize TFT display
  TFT_BLED = 1;                // Turn on TFT backlight
}

// Graphical frame
void DrawFrame(){
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_Line(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("FAT-32  Audio  MP3  Test", 45, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("Mark  Joyce", 232, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

void printItPlay(char *infname)
{
  char txt[] = "Playing:";
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text(txt, 20, 50);
  TFT_Write_Text(infname, 20, 70);
}

void printItEnd(char *infname)
{
  char txt[] = "Playing:";
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text(txt, 20, 50);
  TFT_Write_Text(infname, 20, 70);
  TFT_Write_Text("END", 20, 70);
}

void play(char *infname)
{
 printItPlay(infname);
 MP3_Play(infname);
 printItEnd(infname);
}

void main(){
  int err;
  const int delay = 1000;
  char *myNameIsMark = "hello_my_name_is_mark.mp3";
  //Add more files like this, I have tested these file name lengths.
  /*
  char *iWouldLike = "i_would_like.mp3";
  char *food = "food.mp3";
  char *help = "help.mp3";
  char *yes = "yes.mp3";
  char *no = "no.mp3";
  char *wouldYouLike = "do_you_want_to_play_with_me.mp3";
  */
  
  Init();                      // Initialize MCU
  DrawFrame();                 // Draw graphical frame
  MP3_Init();                  // Start using mp3 codec

  err = FAT32_Init(); // Start SD Card ~ FAT32
  if (err < 0)        //  If Error - No SD card or imcompatable/wrong format inserted.
  {
      TFT_Set_Font(TFT_defaultFont, CL_RED, FO_HORIZONTAL);
      TFT_Write_Text("Please  Insert  SD  Card!", 80, 50);

      while(err < 0)  //  If ERROR ...retry each second
      {
          err = FAT32_Init();
          Delay_ms(1000);
      }
      TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
      TFT_Write_Text("Please  Insert  SD  Card!", 80, 50);
      
  }
  // Play some MP3's  ; look in MP3_routines.c  --->
  play(myNameIsMark);
  Delay_ms(delay);
  /*
  //Add more files and load them here, for example.
  play(iWouldLike);                  // Play mp3 file
  play(food);                  // Play mp3 file
  Delay_ms(delay);
  play(iWouldLike);
  play(help);                // Play mp3 file
  Delay_ms(delay);
  play(yes);                // Play mp3 file
  Delay_ms(delay);
  play(no);                // Play mp3 file
  Delay_ms(delay);
  play(wouldYouLike);                // Play mp3 file
  */

  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("END", 80, 50);
  
}